<?php

/******************************************************************************
 * iSearch - website search engine                                            *
 *                                                                            *
 * Visit the iSearch homepage at http://www.digvid.info/isearch/home.php      *
 *                                                                            *
 * Copyright (C) 2002-2003 Ian Willis. All rights reserved.                   *
 *                                                                            *
 * This script is free to use for non-commercial individual use.              *
 *                                                                            *
 * Use of this script by companies or on commercial web sites requires a      *
 * commercial license. For commercial licence details, please visit:          *
 *     http://www.digvid.info/isearch/commercial.php                          *
 *                                                                            *
 ******************************************************************************/

/* Parse the robots.txt stored in the database */
function isearch_parseRobots($domain, $verbose)
{
    global $isearch_robotsDomains;
    global $isearch_robotsExcludes;
    global $isearch_table_info;

    $isearch_robotsDomains[] = "$domain";

    $alldata = isearch_readFile("http://$domain/robots.txt", $verbose);

    if ($alldata != "")
    {
        $alldata = strtolower($alldata);
        $lines = split("(\r|\n)", $alldata);
        $validUseragent = False;
        $matched = False;

        foreach ($lines as $line)
        {
            $line = ereg_replace("#.*$", "", $line);
            $line = ereg_replace("[[:space:]]+", " ", $line);
            $temp = split(":", $line, 2);
            $field = trim($temp[0]);
            $value = trim($temp[1]);
            if ($field == "user-agent")
            {
                $validUseragent = False;
                $useragents = split(" ", $value);
                foreach ($useragents as $useragent)
                {
                    if (($useragent == "isearch") || (($useragent == "*") && (!$matched)))
                    {
                        $matched = True;
                        $validUseragent = True;
                    }
                }
            }
            else if (($validUseragent) && ($field == "disallow"))
            {
                if ($value == "")
                {
                    /* This is an allow - remove all previous disallows */
                    unset($disallow);
                }
                else
                {
                    $disallow[] = $value;
                }
            }
        }

        if (isset($disallow))
        {
            foreach ($disallow as $temp)
            {
                if ($temp{0} != "/")
                {
                    $temp = "/" . $temp;
                }
                $url = "^http://$domain$temp";
                $url = ereg_replace('\.', '\.', $url);
                $url = ereg_replace('\*', '.*', $url);
                $url = ereg_replace('\?', '\?', $url);
                $url = ereg_replace('\+', '\+', $url);
                $isearch_robotsExcludes[] = $url;
            }
        }
    }

    $robots = implode(" ", $isearch_robotsDomains) . "#" . implode(" ", $isearch_robotsExcludes);

    mysql_query("UPDATE $isearch_table_info SET robots='" . mysql_escape_string($robots) . "' WHERE id='1'");
}


/* Set the id of the next URL to spider */
function isearch_setNextId($currentId)
{
    global $isearch_table_info, $isearch_table_urls;
    global $isearch_nextId;

    /* Work out next ID to index */
    $result = mysql_query ("SELECT id FROM $isearch_table_urls WHERE id > $currentId LIMIT 1");
    if (($result) && ($item = mysql_fetch_object($result)))
    {
        $isearch_nextId = $item->id;
        mysql_query("UPDATE $isearch_table_info SET next_id='$isearch_nextId' WHERE id='1'");

        isearch_log("NextId is $item->id");

        return True;
    }

    return False;
}


/* Clear the iSearch log file */
function isearch_clearLog()
{
    global $isearch_table_info;

    mysql_query("UPDATE $isearch_table_info SET log=''");
}


/* Return the contents of the spider log */
function isearch_getLog()
{
    global $isearch_table_info;

    $log = "";

    $resultInfo = mysql_query ("SELECT log FROM $isearch_table_info");
    if ($resultInfo)
    {
        $isearch_itemInfo = mysql_fetch_object($resultInfo);
        if ($isearch_itemInfo)
        {
            $log = $isearch_itemInfo->log;
        }
    }

    return $log;
}


/* Save the string in the iSearch log file */
function isearch_log($string, $verbose = False)
{
    global $isearch_table_info;

    $log = isearch_getLog() . $string . "<BR>\n";

    mysql_query("UPDATE $isearch_table_info SET log='$log'");

    if ($verbose)
    {
        echo $string . "<BR>\n";
    }
}


/* Clean up a string to make it suitable for storing in search index */
function isearch_cleanString($data)
{
    /* Convert to lower case, doing accented character conversion correctly */
    $data = strtr($data, "ABCDEFGHIJKLMNOPQRSTUVWXYZ" . chr(0x8a) . chr(0x8e) ,
                         "abcdefghijklmnopqrstuvwxyz" . chr(0x9a) . chr(0x9e) );

    /* Strip out all HTML tags */
    $data = strip_tags($data);

    /* Replace some breaking chars with spaces */
    $data = eregi_replace("(:|;|,|\.|-|\?|!)", " ", $data);

    /* Strip out all characters except whitespace numeric and alpha */
    $data = preg_replace('/([^0-9a-z-' . chr(0x9a) . chr(0x9e) . '\s])/e', "", $data);

    /* Compact all white space into a single space character */
    $data = ereg_replace("[[:space:]]+", " ", $data);

    /* Strip white space from beginning and end of the string */
    $data = trim($data);

    return $data;
}


function isearch_readFile($url, $verbose = False)
{
    global $isearch_maxFileSize;
    global $isearch_urlSearch;
    global $isearch_urlReplace;

    if ("$isearch_urlSearch" != "")
    {
        $url = ereg_replace($urlSearch, $urlReplace, $url);
    }

    if (! $isearch_alternative_file_read)
    {
        $alldata = "";
        $fp = @fopen($url, "r");
        if ($fp)
        {
            $alldata = fread($fp, $isearch_maxFileSize);
            fclose($fp);
        }
        return $alldata;
    }

    $recurse = 10;

    while (($recurse > 0) && (eregi("^http://", $url)))
    {
        $host = eregi_replace("^http://", "", $url);
        $host = eregi_replace("/.*", "", $host);
        $temp = explode(":", $host);
        $port = $temp[count($temp)-1];
        if (!ereg("^[0-9]+$", $port));
        {
            $port = 80;
        }
        $host = eregi_replace(":.*$", "", $host);
        $sock = fsockopen($host, $port);

        if (!$sock)
        {
            isearch_log("Unable to open socket", $verbose);
            return "";
        }

        $request = "GET ".$url." HTTP/1.0\r\n";
        $request .= "User-Agent: iSearch/$isearch_version\r\n";
        $request .= "Host: ".$host."\r\n";
        $request .= "Connection: Close\r\n\r\n";

        fputs($sock, $request);

        $header = array();
        $content = "";

        /* Read status line */
        $status = fgets($sock, 1024);

        while (!feof($sock))
        {
            $line = fgets ( $sock, 1024 );
            if($line == "\r\n")
            {
                $content = fread ( $sock, $isearch_maxFileSize );
                break;
            }
            else
            {
                $data = explode(": ",$line, 2);
                if (count($data) == 2)
                {
                    $header[$data[0]] = $data[1];
                }
            }
        }
        fclose($sock);

        /* Check status code:
         * "200"   ; OK
         * "201"   ; Created
         * "202"   ; Accepted
         * "204"   ; No Content
         * "301"   ; Moved Permanently
         * "302"   ; Moved Temporarily
         * "304"   ; Not Modified
         * "400"   ; Bad Request
         * "401"   ; Unauthorized
         * "403"   ; Forbidden
         * "404"   ; Not Found
         * "500"   ; Internal Server Error
         * "501"   ; Not Implemented
         * "502"   ; Bad Gateway
         * "503"   ; Service Unavailable
         */
        $temp = explode(" ", $status, 3);
        if (("$temp[1]" == "301") || ("$temp[1]" == "302"))
        {
            /* Redirection. Get new location */
            $url = $header['Location'];
            if ("$url" == "")
            {
                return "";
            }
            $recurse = $recurse - 1;
            continue;
        }
        else if ($temp[1] >= 300)
        {
            return "";
        }

        return $content;
    }

    return "";
}


/* Spider a single file */
function isearch_indexAFile($verbose = False)
{
    global $isearch_table_info, $isearch_table_urls;
    global $isearch_maxFileSize;
    global $isearch_alternative_file_read;
    global $isearch_followFrames;
    global $isearch_keepCache;
    global $isearch_nextId;

    $moreToDo = False;

    if (isearch_lock())
    {
        $resultUrls = mysql_query ("SELECT * FROM $isearch_table_urls WHERE id='$isearch_nextId'");
        if (($resultUrls) && (mysql_num_rows($resultUrls) > 0))
        {
            $itemUrl = mysql_fetch_object($resultUrls);

            $alldata = isearch_readFile("$itemUrl->url", $verbose);

            $doIt = False;
            if ("$alldata" == "")
            {
                isearch_log("Unable to open URL [$itemUrl->url]", $verbose);
            }
            else
            {
                $size = strlen($alldata);
                $sig = md5($alldata);

                /* Check whether this page has changed since last spidered */
                if (($itemUrl->sig == $sig) && ($itemUrl->size == $size))
                {
                    isearch_log("URL not changed since last spidering [$itemUrl->url]", $verbose);
                }
                else
                {
                    /* Page has changed - now look for a duplicate */
                    $resultSig = mysql_query ("SELECT * FROM $isearch_table_urls WHERE sig='$sig' AND size='$size' AND NOT url='" . $item->url . "'");
                    if (($resultSig) && (mysql_num_rows($resultSig) > 0))
                    {
                        isearch_log("  Duplicate URL - will not be processed [$itemUrl->url]", $verbose);
                    }
                    else
                    {
                        /* No duplicates found. */
                        $doIt = True;
                    }
                }
            }

            if (! $doIt)
            {
                mysql_query("UPDATE $isearch_table_urls SET found='0' WHERE id='$isearch_nextId'");
            }
            else
            {
                isearch_log("Processing URL [$itemUrl->url]", $verbose);

                if ($isearch_keepCache)
                {
                    $cache = $alldata;
                }
                else
                {
                    $cache = "";
                }

                /* Strip out all control characters - replace with whitespace */
                $alldata = ereg_replace("[[:cntrl:]]", " ", $alldata);

                /* Compact all white space into a single space character */
                $alldata = ereg_replace("[[:space:]]+", " ", $alldata);

                $tdata = spliti("<body", $alldata, 2);

                $headdata = $tdata[0];
                $bodydata = "<body" . $tdata[1];

                /* Strip out all HTML tags except special ones */
                $headdata = strip_tags($headdata, '<meta><title><base>');

                $keywords = "";
                $description = "";
                $noIndex = False;
                $noFollow = False;
                $noArchive = False;
                $metas = spliti("<meta ", "x" . $headdata);
                for ($i = 1; $i < count($metas); $i++)
                {
                    $titem = spliti(">", $metas[$i], 2);
                    $item = ereg_replace("<meta ", "", $titem[0]);

                    $metaItems = spliti("name=['\"]", "x" . $item, 2);
                    $metaItems = spliti("['\"]", $metaItems[1], 3);
                    $metaName = strtolower($metaItems[0]);

                    $metaItems = spliti("content=['\"]", "x" . $item, 2);
                    $metaItems = spliti("['\"]", $metaItems[1], 3);
                    $metaContent = $metaItems[0];

                    if ($metaName == "keywords")
                    {
                        $keywords = ereg_replace(","," ", $metaContent);
                    }
                    else if ($metaName == "description")
                    {
                        $description = $metaContent;
                    }
                    else if ($metaName == "robots")
                    {
                        if (eregi("noindex", $metaContent))
                        {
                            $noIndex = True;
                        }
                        if (eregi("nofollow", $metaContent))
                        {
                            $noFollow = True;
                        }
                        if (eregi("noarchive", $metaContent))
                        {
                            $noArchive = True;
                            $cache = "";
                        }
                    }
                }

                /* Find a "<BASE HREF=" tag in the document head */
                $base = $itemUrl->url;
                $tdata1 = spliti("<base[^>]*href=", 'x' . $headdata);
                if (count($tdata1) > 1)
                {
                    $item = ereg_replace("[>[:space:]].*$","", $tdata1[1]);
                    $base = ereg_replace("[\"']","", $item);    // Strip quotes '
                }

                if (!$noIndex)
                {
                    $title1 = spliti("<title>", "x" . $headdata, 3);
                    if (count($title1) >= 2)
                    {
                        /* Found at least 1 <TITLE> */
                        $title2 = spliti("</title>", $title1[1], 3);
                        if (count($title2) >= 2)
                        {
                            /* Found at least 1 </TITLE> */
                            $title = $title2[0];

                            /* Many webmasters add keywords into their page titles
                             * for better search engine placement. Add all words
                             * from the page title into the keywords list.
                             */
                            $keywords .= " " . $title;
                        }
                    }
                }

                /* Replace breaking tags and other special chars with spaces */
                $bodydata = eregi_replace("(<hr(>| [^>]*>)|<br(>| [^>]*>)|<p(>| [^>]*>)|<td(>| [^>]*>)|<th(>| [^>]*>)|&nbsp;)", " ", $bodydata);

                $tdata1 = split("<!-- ISEARCH_BEGIN_INDEX -->", "x" . $bodydata);
                if (count($tdata1) > 1)
                {
                    /* At least 1 found. */

                    $bodydata = "";

                    /* Check for an END_INDEX before the first BEGIN_INDEX */
                    $tdata2 = split("<!-- ISEARCH_END_INDEX -->", $tdata1[0]);
                    if (count($tdata2) > 1)
                    {
                        /* And END_INDEX was found. Add anything before it into the bodydata */
                        $bodydata .= $tdata2[0];
                    }

                    for ($i = 1; $i < count($tdata1); $i++)
                    {
                        $tdata2 = split("<!-- ISEARCH_END_INDEX -->", $tdata1[$i]);
                        $bodydata .= $tdata2[0];
                    }
                }

                /* Strip out HTML comments. This will remove JavaScript, which
                 * can confuse the strip_tags function.
                 */
                $bodydata = preg_replace('/<!--.+?-->/','',$bodydata);

                /* Strip out all HTML tags except special ones */
                $bodydata = strip_tags($bodydata, '<a><h1><h2><h3><h4><h5>');

                if (!$noFollow)
                {
                    /* Find any links */
                    $tdata1 = spliti("<a[^>]*href=", 'x' . $bodydata);
                    for ($i = 1; $i < count($tdata1); $i++)
                    {
                        $item = ereg_replace("[>[:space:]].*$","", $tdata1[$i]);
                        $url = ereg_replace("[\"']","", $item);    // Strip quotes '
                        isearch_addUrl($url, $base, $verbose, $isearch_nextId);
                    }

                    if ($isearch_followFrames)
                    {
                        /* Search for frames, and add the referenced docs to the URL list */
                        $frameset = "";
                        $tdata1 = spliti("<frameset", $alldata);
                        for ($i = 1; $i < count($tdata1); $i++)
                        {
                            $tdata2 = spliti("</frameset>", $tdata1[$i], 3);
                            if (count($tdata2) >= 2)
                            {
                                /* Found at least 1 </frameset> tag. Search for <frame src=> tags */
                                $tdata3 = spliti("<frame[^>]*src=", 'x' . $tdata2[0]);
                                for ($i = 1; $i < count($tdata3); $i++)
                                {
                                    $tdata4 = spliti(">", $tdata3[$i], 2);
                                    $item = ereg_replace("[[:space:]].*$","", $tdata4[0]);
                                    $url = ereg_replace("[\"']","", $item);    // Strip quotes '
                                    isearch_addUrl($url, $base, $verbose, $isearch_nextId);
                                }
                            }
                        }
                    }
                }

                if (!$noIndex)
                {
                    /* Convert to lower case */
                    $bodydata = strtolower($bodydata);

                    $bigwords = "";
                    $tdata1 = split("<h[1-5]>", "x" . $bodydata);
                    for ($i = 1; $i < count($tdata1); $i++)
                    {
                        $tdata2 = split("</h[1-5]>", $tdata1[$i], 3);
                        if (count($tdata2) >= 2)
                        {
                            /* Found at least 1 </h?> tag */
                            $bigwords .= " " . $tdata2[0];
                        }
                    }

                    /* Translate from HTML to ASCII */
                    $trans = get_html_translation_table(HTML_ENTITIES);
                    $trans = array_flip($trans);
                    $bodydata = strtr($bodydata, $trans);
                    $bigwords = strtr($bigwords, $trans);
                    $keywords = strtr($keywords, $trans);

                    /* Strip out unwanted characters from the strings that we
                     * search on
                     */
                    $bodydata = isearch_cleanString($bodydata);
                    $bigwords = isearch_cleanString($bigwords);
                    $keywords = isearch_cleanString($keywords);

                    /* Add keywords to the list of words for search matches.
                     * Seperate from the body text with a # symbol.
                     */
                    $bodydata .= " # $keywords $description";
                    mysql_query("UPDATE $isearch_table_urls SET " .
                                    "keywords='" . mysql_escape_string($keywords) . "', " .
                                    "title='" . mysql_escape_string($title) . "', " .
                                    "bigwords='" . mysql_escape_string($bigwords) . "', " .
                                    "description='" . mysql_escape_string($description) . "', " .
                                    "words='" . mysql_escape_string($bodydata) . "', " .
                                    "cache='" . mysql_escape_string($cache) . "', " .
                                    "size='$size', " .
                                    "base='" . mysql_escape_string($base) . "', " .
                                    "sig='" . mysql_escape_string($sig) . "', " .
                                    "found='1' WHERE id='$isearch_nextId'");
                }
            }
        }
        else
        {
            isearch_log("Unable to retrieve URL index $isearch_nextId", $verbose);
        }

        $moreToDo = isearch_setNextId($isearch_nextId);
        if (!$moreToDo)
        {
            isearch_log("Indexing completed.", $verbose);

            /* Indexing has completed */
            $now = time();
            mysql_query("UPDATE $isearch_table_info SET next_id='-1', last_update='$now' WHERE id='1'");

            if ($verbose)
            {
                $result = mysql_query ("SELECT url FROM $isearch_table_urls WHERE found='0'");
                if (($result) && (mysql_num_rows($result) > 0))
                {
                    isearch_log("Deleting the following URLs:", $verbose);
                    while ($item = mysql_fetch_object($result))
                    {
                        isearch_log("$item->url", $verbose);
                    }
                }
            }
            /* Delete any unfound references */
            mysql_query("DELETE FROM $isearch_table_urls WHERE found='0'");

            /* Update referrer_id fields */
            $result = mysql_query ("SELECT id, temp_referrer_id FROM $isearch_table_urls");
            if (($result) && (mysql_num_rows($result) > 0))
            {
                mysql_query ("LOCK TABLES $isearch_table_urls WRITE");
                while ($item = mysql_fetch_object($result))
                {
                    mysql_query("UPDATE $isearch_table_urls SET referrer_id='$item->temp_referrer_id' WHERE id='$item->id'");
                }
                mysql_query ("UNLOCK TABLES");
            }

            /* Optimize the table */
            mysql_query("OPTIMIZE TABLE $isearch_table_urls");
        }

        isearch_unlock();
    }
    else
    {
        isearch_log("Unable to get lock", $verbose);
    }

    return $moreToDo;
}


/* Glue the fragments from a parse_url together. */
function glue_url($parsed)
{
    if (! is_array($parsed))
    {
        return false;
    }

    if (isset($parsed['scheme']))
    {
        $sep = (strtolower($parsed['scheme']) == 'mailto' ? ':' : '://');
        $uri = $parsed['scheme'] . $sep;
    }
    else
    {
        $uri = '';
    }

    if (isset($parsed['pass']))
    {
        $uri .= "$parsed[user]:$parsed[pass]@";
    }
    elseif (isset($parsed['user']))
    {
        $uri .= "$parsed[user]@";
    }

    if (isset($parsed['host']))
    {
        $uri .= $parsed['host'];
    }
    if (isset($parsed['port']))
    {
        $uri .= ":$parsed[port]";
    }
    if (isset($parsed['path']))
    {
        $uri .= $parsed['path'];
    }
    if (isset($parsed['query']))
    {
        $uri .= "?$parsed[query]";
    }
    if (isset($parsed['fragment']))
    {
        $uri .= "#$parsed[fragment]";
    }

    return $uri;
}


/* Add a URL to the search index */
function isearch_addUrl($newUrl, $relativeToUrl, $verbose, $referrer_id)
{
    global $isearch_allowedFileExtensions;
    global $isearch_allowDirectories;
    global $isearch_allowedUrlsBeginning;
    global $isearch_allowedUrls;
    global $isearch_disallowedUrlsBeginning;
    global $isearch_disallowedUrls;
    global $isearch_table_urls;
    global $isearch_removeGetVars;
    global $isearch_wwwOption;
    global $isearch_stripDefaults;
    global $isearch_robotsDomains;
    global $isearch_robotsExcludes;

    isearch_log("&nbsp;&nbsp;Checking URL [$newUrl $relativeToUrl]");

    /* Convert to absolute reference */
    if ((eregi("^http:", $newUrl)) || (eregi("^ftp:", $newUrl)) || (eregi("^file:", $newUrl)))
    {
        $absoluteUrl = $newUrl;
    }
    else if (eregi("^([a-z]+):", $newUrl))
    {
        isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Rejecting unsupported URL format [$newUrl]");
        return;
    }
    else
    {
        if (ereg("^/", $newUrl))
        {
            /* New URL begins with a slash. It is within the site */

            /* Extract the site URL */
            $siteParts = @parse_url($relativeToUrl);
            if ((!isset($siteParts['scheme'])) || (!isset($siteParts['host'])))
            {
                /* Unable to parse relativeToUrl */
                isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Unable to parse relativeToUrl [$relativeToUrl]");
                return;
            }
            unset($siteParts['path']);
            unset($siteParts['query']);
            unset($siteParts['fragment']);
            $siteUrl = glue_url($siteParts);

            $absoluteUrl = $siteUrl . $newUrl;
        }
        else
        {
            /* A relative reference (must be within this site) */
            $temp = explode("/", $relativeToUrl);
            unset($temp[count($temp)-1]);   /* Remove anything following the last slash */
            $absoluteUrl = implode("/", $temp) . "/" . $newUrl;
            $absoluteUrl = eregi_replace("/\.$", "", $absoluteUrl);             /* Remove ending "/." */
            $absoluteUrl = eregi_replace("/\./", "/", $absoluteUrl);            /* Remove any "." references */
            $absoluteUrl = eregi_replace("//", "/", $absoluteUrl);              /* Remove excess slashes */

            /* Resolve any ".." references */
            $temp = explode("/", $absoluteUrl);
            for ($i = 2; $i < count($temp); $i++)
            {
                if (($temp[$i] == "..") && ($i > 1))
                {
                    for ($j = $i + 1; $j < count($temp); $j ++)
                    {
                        $temp[$j-2] = $temp[$j];
                    }
                    unset($temp[count($temp)-1]);
                    unset($temp[count($temp)-1]);
                    $i = $i - 2;
                }
            }
            $absoluteUrl = implode("/", $temp);
            $absoluteUrl = eregi_replace("^http:/", "http://", $absoluteUrl);
        }
    }

    /* Split the absolute URL into component parts */
    $absoluteParts = @parse_url($absoluteUrl);

    if ((!isset($absoluteParts['scheme'])) || (!isset($absoluteParts['host'])))
    {
        /* Unable to parse relativeToUrl */
        isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Unable to parse absoluteUrl [$absoluteUrl]");
        return;
    }

    /* Remove any anchor reference in the URL */
    unset($absoluteParts['fragment']);

    /* Split the GET variables and remove unnecessary ones */
    if (isset($absoluteParts['query']))
    {
        $temp = split("(&amp;|&)", $absoluteParts['query']);

        foreach (array_keys($temp) as $key)
        {
            $varName = eregi_replace("=.*$", "", $temp[$key]);
            if (in_array($varName, $isearch_removeGetVars))
            {
                unset($temp[$key]);
            }
        }

        $absoluteParts['query'] = implode("&amp;", $temp);

        if ($absoluteParts['query'] == "")
        {
            unset($absoluteParts['query']);
        }
    }

    /* Handle the www subdomain according to the value of $isearch_wwwOption.
     *  1 = Leave as is
     *  2 = Strip www subdomains
     *  3 = Add www subdomain
     */
    if ($isearch_wwwOption == 2)
    {
        $absoluteParts['host'] = eregi_replace("^www\.", "", $absoluteParts['host']);
    }
    else if ($isearch_wwwOption == 3)
    {
        if (!eregi("^www\.", $absoluteParts['host']))
        {
            $absoluteParts['host'] = "www." . $absoluteParts['host'];
        }
    }

    $filePath = $absoluteParts['path'];
    $fileName = eregi_replace(".*/", "", $filePath);
    $fileExtension = eregi_replace('.*\.', "", $fileName);

    /* Glue URL parts together again */
    $absoluteUrl = glue_url($absoluteParts);

    isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Absolute URL [$absoluteUrl]");

    $allowed = False;
    if ($fileExtension == "")
    {
        /* Treat no file extension as a directory. Check whether there is a
         * trailing slash on the URL and add it if necessary.
         */
        if (!eregi("/$", $absoluteUrl))
        {
            $absoluteUrl .= "/";
        }

        if ($isearch_allowDirectories)
        {
            $allowed = True;
        }
    }
    else
    {
        /* Check whether this is a default file name, and strip it if so. */
        foreach ($isearch_stripDefaults as $item)
        {
            if (("$item" != "") && ("$item" == $fileName))
            {
                $absoluteUrl = ereg_replace("[^/]*$", "", $absoluteUrl);
                $allowed = True;
                break;
            }
        }

        /* Check whether this file extension is allowed */
        if (in_array($fileExtension, $isearch_allowedFileExtensions))
        {
            $allowed = True;
        }
    }

    if (!$allowed)
    {
        isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Rejecting URL - it is not in allowed file extensions [$absoluteUrl]");
    }
    else
    {
        $allowed = False;
        /* Check that this URL is in the allowed list */
        foreach ($isearch_allowedUrlsBeginning as $item)
        {
            if (("$item" != "") && ($item == substr($absoluteUrl, 0, strlen($item))))
            {
                $allowed = True;
                break;
            }
        }

        foreach ($isearch_allowedUrls as $item)
        {
            if (("$item" != "") && (eregi($item, $absoluteUrl)))
            {
                $allowed = True;
                break;
            }
        }

        if (!$allowed)
        {
            isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Rejecting URL - it  is not in allowed URL list [$absoluteUrl]");
        }
        else
        {
            if (($absoluteParts['scheme'] == "http") && (!in_array($absoluteParts['host'], $isearch_robotsDomains)))
            {
                /* Parse the robots.txt for this domain */
                isearch_parseRobots($absoluteParts['host'], $verbose);
            }

            /* Check that this URL is not in the disallowed list */
            foreach ($isearch_disallowedUrlsBeginning as $item)
            {
                if (("$item" != "") && ($item == substr($absoluteUrl, 0, strlen($item))))
                {
                    $allowed = False;
                    isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Rejecting URL - it is in disallowed URL beginning list [$absoluteUrl]");
                    break;
                }
            }

            if ($allowed)
            {
                foreach ($isearch_disallowedUrls as $item)
                {
                    if (("$item" != "") && (eregi($item, $absoluteUrl)))
                    {
                        $allowed = False;
                        isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Rejecting URL - it is in disallowed URL regexp list [$absoluteUrl]");
                        break;
                    }
                }
            }

            if ($allowed)
            {
                /* Check the robots.txt excludes list */
                foreach ($isearch_robotsExcludes as $item)
                {
                    if (("$item" != "") && (eregi($item, $absoluteUrl)))
                    {
                        $allowed = False;
                        isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;Rejecting URL - it is disallowed by robots.txt [$absoluteUrl]");
                        break;
                    }
                }
            }
        }
    }

    if ($allowed)
    {
        /* Allowed URL - Check whether it's already in the table of known URLs */
        $result = mysql_query ("SELECT found, id FROM $isearch_table_urls WHERE url='$absoluteUrl'");
        if (($result) && (mysql_num_rows($result) > 0))
        {
            /* Already in database */
            isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;URL allowed, but already in database [$absoluteUrl]");
            if ($item = mysql_fetch_object($result))
            {
                if ((!$item->found) && ($item->id != $referrer_id))
                {
                    mysql_query("UPDATE $isearch_table_urls SET temp_referrer_id='$referrer_id' WHERE url='$absoluteUrl'");
                }
            }
        }
        else
        {
            /* Not found - add it */
            mysql_query ("INSERT INTO $isearch_table_urls (url, temp_referrer_id) VALUES ('$absoluteUrl', '$referrer_id')");
            isearch_log("&nbsp;&nbsp;&nbsp;&nbsp;URL allowed, and added [$absoluteUrl]");
        }
    }
}


/* Reset the search index to allow site to be re-spidered */
function isearch_reset()
{
    global $isearch_startUrls;
    global $isearch_table_urls;
    global $isearch_robotsDomains;
    global $isearch_robotsExcludes;

    if (isearch_lock())
    {
        /* Check that all start URLs are in database. Add them if not */
        foreach ($isearch_startUrls as $url)
        {
            /* Find whether this URL refers to a directory */
            $urlParts = @parse_url($url);
            if ((!isset($urlParts['scheme'])) || (!isset($urlParts['host'])))
            {
                /* Unable to parse relativeToUrl */
                isearch_log("Unable to parse start URL [$url]");
            }
            else
            {
                $fileExtension = eregi_replace("\?.*$", "", $urlParts['path']);
                $fileExtension = eregi_replace(".*/", "", $fileExtension);
                $fileExtension = eregi_replace(".*\.", "", $fileExtension);

                if ($fileExtension == "")
                {
                    /* Treat no file extension as a directory. Check whether there is a
                     * trailing slash on the URL and add it if necessary.
                     */
                    if (!eregi("/$", $url))
                    {
                        $url .= "/";
                    }
                }

                $result = mysql_query ("SELECT url FROM $isearch_table_urls WHERE url='$url'");
                if (($result) && (mysql_num_rows($result) > 0))
                {
                    /* Already in database */
                    mysql_query("UPDATE $isearch_table_urls SET temp_referrer_id='-1' WHERE url='$url'");
                    isearch_log("Start URL already in database [$url]");
                }
                else
                {
                    /* Not found - add it */
                    mysql_query ("INSERT INTO $isearch_table_urls (url, temp_referrer_id) VALUES ('$url', '-1')");
                    isearch_log("Added start URL [$url]");
                }
            }
        }

        /* Find the id of the first entry in the database */
        isearch_setNextId(-1);

        /* Mark all entries as not found */
        mysql_query("UPDATE $isearch_table_urls SET found='0'");

        isearch_clearLog();

        /* Reset the robots.txt exclude list */
        $isearch_robotsDomains = array();
        $isearch_robotsExcludes = array();
        mysql_query("UPDATE $isearch_table_info SET robots='#' WHERE id='1'");

        /* Release the info lock */
        isearch_unlock();
    }
}

?>
